@@ -3,7 +3,7 @@ Redirecting old docs to new location
 ====================================
 
 If an rst file is moved or its content subsumed in a different file, it
-is desireable to redirect the old file to the new or existing file. This
+is desirable to redirect the old file to the new or existing file. This
 extension enables this with a simple html refresh.
 
 For example suppose ``doc/topic/old-page.rst`` is removed and its content
@@ -15,8 +15,10 @@ directive in ``doc/topic/new-page.rst``::
 This creates in the build directory a file ``build/html/topic/old-page.html``
 that contains a relative refresh::
 
+    <!DOCTYPE html>
     <html>
       <head>
+        <meta charset="utf-8">
         <meta http-equiv="refresh" content="0; url=new-page.html">
       </head>
     </html>
@@ -32,13 +34,16 @@ full path::
 
 from pathlib import Path
 from docutils.parsers.rst import Directive
+from sphinx.domains import Domain
 from sphinx.util import logging
 
 logger = logging.getLogger(__name__)
 
 
-HTML_TEMPLATE = """<html>
+HTML_TEMPLATE = """<!DOCTYPE html>
+<html>
   <head>
+    <meta charset="utf-8">
     <meta http-equiv="refresh" content="0; url={v}">
   </head>
 </html>
@@ -48,24 +53,53 @@ HTML_TEMPLATE = """<html>
 def setup(app):
     RedirectFrom.app = app
     app.add_directive("redirect-from", RedirectFrom)
+    app.add_domain(RedirectFromDomain)
     app.connect("build-finished", _generate_redirects)
 
+    metadata = {'parallel_read_safe': True}
+    return metadata
+
+
+class RedirectFromDomain(Domain):
+    """
+    The sole purpose of this domain is a parallel_read_safe data store for the
+    redirects mapping.
+    """
+    name = 'redirect_from'
+    label = 'redirect_from'
+
+    @property
+    def redirects(self):
+        """The mapping of the redirects."""
+        return self.data.setdefault('redirects', {})
+
+    def clear_doc(self, docnames):
+        self.redirects.clear()
+
+    def merge_domaindata(self, docnames, otherdata):
+        for src, dst in otherdata['redirects'].items():
+            if src not in self.redirects:
+                self.redirects[src] = dst
+            elif self.redirects[src] != dst:
+                raise ValueError(
+                    f"Inconsistent redirections from {src} to "
+                    f"{self.redirects[src]} and {otherdata['redirects'][src]}")
+
 
 class RedirectFrom(Directive):
     required_arguments = 1
-    redirects = {}
 
     def run(self):
         redirected_doc, = self.arguments
         env = self.app.env
-        builder = self.app.builder
+        domain = env.get_domain('redirect_from')
         current_doc = env.path2doc(self.state.document.current_source)
         redirected_reldoc, _ = env.relfn2path(redirected_doc, current_doc)
-        if redirected_reldoc in self.redirects:
+        if redirected_reldoc in domain.redirects:
             raise ValueError(
                 f"{redirected_reldoc} is already noted as redirecting to "
-                f"{self.redirects[redirected_reldoc]}")
-        self.redirects[redirected_reldoc] = current_doc
+                f"{domain.redirects[redirected_reldoc]}")
+        domain.redirects[redirected_reldoc] = current_doc
         return []
 
 
@@ -73,15 +107,15 @@ def _generate_redirects(app, exception):
     builder = app.builder
     if builder.name != "html" or exception:
         return
-    for k, v in RedirectFrom.redirects.items():
+    for k, v in app.env.get_domain('redirect_from').redirects.items():
         p = Path(app.outdir, k + builder.out_suffix)
         html = HTML_TEMPLATE.format(v=builder.get_relative_uri(k, v))
         if p.is_file():
             if p.read_text() != html:
-                logger.warning(f'A redirect-from directive is trying to '
-                               f'create {p}, but that file already exists '
-                               f'(perhaps you need to run "make clean")')
+                logger.warning('A redirect-from directive is trying to '
+                               'create %s, but that file already exists '
+                               '(perhaps you need to run "make clean")', p)
         else:
-            logger.info(f'making refresh html file: {k} redirect to {v}')
+            logger.info('making refresh html file: %s redirect to %s', k, v)
             p.parent.mkdir(parents=True, exist_ok=True)
-            p.write_text(html)
+            p.write_text(html, encoding='utf-8')
