@@ -14,7 +14,7 @@ import numpy as np
 from PIL import Image
 
 import matplotlib as mpl
-from matplotlib import _api, cbook, font_manager as fm
+from matplotlib import cbook, font_manager as fm
 from matplotlib.backend_bases import (
      _Backend, FigureCanvasBase, FigureManagerBase, RendererBase)
 from matplotlib.backends.backend_mixed import MixedModeRenderer
@@ -24,9 +24,9 @@ from matplotlib.path import Path
 from matplotlib import _path
 from matplotlib.transforms import Affine2D, Affine2DBase
 
+
 _log = logging.getLogger(__name__)
 
-backend_version = mpl.__version__
 
 # ----------------------------------------------------------------------
 # SimpleXMLWriter class
@@ -66,11 +66,6 @@ backend_version = mpl.__version__
 # --------------------------------------------------------------------
 
 
-@_api.deprecated("3.6", alternative="a vendored copy of _escape_cdata")
-def escape_cdata(s):
-    return _escape_cdata(s)
-
-
 def _escape_cdata(s):
     s = s.replace("&", "&amp;")
     s = s.replace("<", "&lt;")
@@ -81,21 +76,11 @@ def _escape_cdata(s):
 _escape_xml_comment = re.compile(r'-(?=-)')
 
 
-@_api.deprecated("3.6", alternative="a vendored copy of _escape_comment")
-def escape_comment(s):
-    return _escape_comment.sub(s)
-
-
 def _escape_comment(s):
     s = _escape_cdata(s)
     return _escape_xml_comment.sub('- ', s)
 
 
-@_api.deprecated("3.6", alternative="a vendored copy of _escape_attrib")
-def escape_attrib(s):
-    return _escape_attrib(s)
-
-
 def _escape_attrib(s):
     s = s.replace("&", "&amp;")
     s = s.replace("'", "&apos;")
@@ -111,17 +96,12 @@ def _quote_escape_attrib(s):
             '"' + _escape_attrib(s) + '"')
 
 
-@_api.deprecated("3.6", alternative="a vendored copy of _short_float_fmt")
-def short_float_fmt(x):
-    return _short_float_fmt(x)
-
-
 def _short_float_fmt(x):
     """
     Create a short string representation of a float, which is %f
     formatting with trailing zeros and the decimal point removed.
     """
-    return '{0:f}'.format(x).rstrip('0').rstrip('.')
+    return f'{x:f}'.rstrip('0').rstrip('.')
 
 
 class XMLWriter:
@@ -177,12 +157,12 @@ class XMLWriter:
         self.__data = []
         self.__tags.append(tag)
         self.__write(self.__indentation[:len(self.__tags) - 1])
-        self.__write("<%s" % tag)
+        self.__write(f"<{tag}")
         for k, v in {**attrib, **extra}.items():
             if v:
                 k = _escape_cdata(k)
                 v = _quote_escape_attrib(v)
-                self.__write(' %s=%s' % (k, v))
+                self.__write(f' {k}={v}')
         self.__open = 1
         return len(self.__tags) - 1
 
@@ -197,7 +177,7 @@ class XMLWriter:
         """
         self.__flush()
         self.__write(self.__indentation[:len(self.__tags)])
-        self.__write("<!-- %s -->\n" % _escape_comment(comment))
+        self.__write(f"<!-- {_escape_comment(comment)} -->\n")
 
     def data(self, text):
         """
@@ -220,11 +200,12 @@ class XMLWriter:
         tag
             Element tag.  If given, the tag must match the start tag.  If
             omitted, the current element is closed.
+        indent : bool, default: True
         """
         if tag:
-            assert self.__tags, "unbalanced end(%s)" % tag
+            assert self.__tags, f"unbalanced end({tag})"
             assert _escape_cdata(tag) == self.__tags[-1], \
-                "expected end(%s), got %s" % (self.__tags[-1], tag)
+                f"expected end({self.__tags[-1]}), got {tag}"
         else:
             assert self.__tags, "unbalanced end()"
         tag = self.__tags.pop()
@@ -236,7 +217,7 @@ class XMLWriter:
             return
         if indent:
             self.__write(self.__indentation[:len(self.__tags)])
-        self.__write("</%s>\n" % tag)
+        self.__write(f"</{tag}>\n")
 
     def close(self, id):
         """
@@ -276,25 +257,15 @@ def _generate_transform(transform_list):
             continue
         if type == 'matrix' and isinstance(value, Affine2DBase):
             value = value.to_values()
-        parts.append('%s(%s)' % (
+        parts.append('{}({})'.format(
             type, ' '.join(_short_float_fmt(x) for x in value)))
     return ' '.join(parts)
 
 
-@_api.deprecated("3.6")
-def generate_transform(transform_list=None):
-    return _generate_transform(transform_list or [])
-
-
 def _generate_css(attrib):
     return "; ".join(f"{k}: {v}" for k, v in attrib.items())
 
 
-@_api.deprecated("3.6")
-def generate_css(attrib=None):
-    return _generate_css(attrib or {})
-
-
 _capstyle_d = {'projecting': 'square', 'butt': 'butt', 'round': 'round'}
 
 
@@ -345,9 +316,9 @@ class RendererSVG(RendererBase):
         svgwriter.write(svgProlog)
         self._start_id = self.writer.start(
             'svg',
-            width='%spt' % str_width,
-            height='%spt' % str_height,
-            viewBox='0 0 %s %s' % (str_width, str_height),
+            width=f'{str_width}pt',
+            height=f'{str_height}pt',
+            viewBox=f'0 0 {str_width} {str_height}',
             xmlns="http://www.w3.org/2000/svg",
             version="1.1",
             attrib={'xmlns:xlink': "http://www.w3.org/1999/xlink"})
@@ -410,7 +381,7 @@ class RendererSVG(RendererBase):
             # See https://reproducible-builds.org/specs/source-date-epoch/
             date = os.getenv("SOURCE_DATE_EPOCH")
             if date:
-                date = datetime.datetime.utcfromtimestamp(int(date))
+                date = datetime.datetime.fromtimestamp(int(date), datetime.timezone.utc)
                 metadata['Date'] = date.replace(tzinfo=UTC).isoformat()
             else:
                 metadata['Date'] = datetime.datetime.today().isoformat()
@@ -499,7 +470,7 @@ class RendererSVG(RendererBase):
         m = hashlib.sha256()
         m.update(salt.encode('utf8'))
         m.update(str(content).encode('utf8'))
-        return '%s%s' % (type, m.hexdigest()[:10])
+        return f'{type}{m.hexdigest()[:10]}'
 
     def _make_flip_transform(self, transform):
         return transform + Affine2D().scale(1, -1).translate(0, self.height)
@@ -573,7 +544,7 @@ class RendererSVG(RendererBase):
         forced_alpha = gc.get_forced_alpha()
 
         if gc.get_hatch() is not None:
-            attrib['fill'] = "url(#%s)" % self._get_hatch(gc, rgbFace)
+            attrib['fill'] = f"url(#{self._get_hatch(gc, rgbFace)})"
             if (rgbFace is not None and len(rgbFace) == 4 and rgbFace[3] != 1.0
                     and not forced_alpha):
                 attrib['fill-opacity'] = _short_float_fmt(rgbFace[3])
@@ -666,7 +637,7 @@ class RendererSVG(RendererBase):
             self.writer.start('g', id=gid)
         else:
             self._groupd[s] = self._groupd.get(s, 0) + 1
-            self.writer.start('g', id="%s_%d" % (s, self._groupd[s]))
+            self.writer.start('g', id=f"{s}_{self._groupd[s]:d}")
 
     def close_group(self, s):
         # docstring inherited
@@ -729,7 +700,7 @@ class RendererSVG(RendererBase):
 
         writer.start('g', **self._get_clip_attrs(gc))
         trans_and_flip = self._make_flip_transform(trans)
-        attrib = {'xlink:href': '#%s' % oid}
+        attrib = {'xlink:href': f'#{oid}'}
         clip = (0, 0, self.width*72, self.height*72)
         for vertices, code in path.iter_segments(
                 trans_and_flip, clip=clip, simplify=False):
@@ -769,7 +740,7 @@ class RendererSVG(RendererBase):
                 master_transform, paths, all_transforms)):
             transform = Affine2D(transform.get_matrix()).scale(1.0, -1.0)
             d = self._convert_path(path, transform, simplify=False)
-            oid = 'C%x_%x_%s' % (
+            oid = 'C{:x}_{:x}_{}'.format(
                 self._path_collection_id, i, self._make_id('', d))
             writer.element('path', id=oid, d=d)
             path_codes.append(oid)
@@ -786,7 +757,7 @@ class RendererSVG(RendererBase):
             if clip_attrs:
                 writer.start('g', **clip_attrs)
             attrib = {
-                'xlink:href': '#%s' % path_id,
+                'xlink:href': f'#{path_id}',
                 'x': _short_float_fmt(xo),
                 'y': _short_float_fmt(self.height - yo),
                 'style': self._get_style(gc0, rgbFace)
@@ -801,7 +772,9 @@ class RendererSVG(RendererBase):
 
     def draw_gouraud_triangle(self, gc, points, colors, trans):
         # docstring inherited
+        self._draw_gouraud_triangle(gc, points, colors, trans)
 
+    def _draw_gouraud_triangle(self, gc, points, colors, trans):
         # This uses a method described here:
         #
         #   http://www.svgopen.org/2005/papers/Converting3DFaceToSVG/index.html
@@ -868,7 +841,7 @@ class RendererSVG(RendererBase):
 
             writer.start(
                 'linearGradient',
-                id="GR%x_%d" % (self._n_gradients, i),
+                id=f"GR{self._n_gradients:x}_{i:d}",
                 gradientUnits="userSpaceOnUse",
                 x1=_short_float_fmt(x1), y1=_short_float_fmt(y1),
                 x2=_short_float_fmt(xb), y2=_short_float_fmt(yb))
@@ -910,20 +883,20 @@ class RendererSVG(RendererBase):
         writer.element(
             'path',
             attrib={'d': dpath,
-                    'fill': 'url(#GR%x_0)' % self._n_gradients,
+                    'fill': f'url(#GR{self._n_gradients:x}_0)',
                     'shape-rendering': "crispEdges"})
 
         writer.element(
             'path',
             attrib={'d': dpath,
-                    'fill': 'url(#GR%x_1)' % self._n_gradients,
+                    'fill': f'url(#GR{self._n_gradients:x}_1)',
                     'filter': 'url(#colorAdd)',
                     'shape-rendering': "crispEdges"})
 
         writer.element(
             'path',
             attrib={'d': dpath,
-                    'fill': 'url(#GR%x_2)' % self._n_gradients,
+                    'fill': f'url(#GR{self._n_gradients:x}_2)',
                     'filter': 'url(#colorAdd)',
                     'shape-rendering': "crispEdges"})
 
@@ -936,7 +909,7 @@ class RendererSVG(RendererBase):
         self.writer.start('g', **self._get_clip_attrs(gc))
         transform = transform.frozen()
         for tri, col in zip(triangles_array, colors_array):
-            self.draw_gouraud_triangle(gc, tri, col, transform)
+            self._draw_gouraud_triangle(gc, tri, col, transform)
         self.writer.end('g')
 
     def option_scale_image(self):
@@ -977,8 +950,7 @@ class RendererSVG(RendererBase):
             if self.basename is None:
                 raise ValueError("Cannot save image data to filesystem when "
                                  "writing SVG to an in-memory buffer")
-            filename = '{}.image{}.png'.format(
-                self.basename, next(self._image_counter))
+            filename = f'{self.basename}.image{next(self._image_counter)}.png'
             _log.info('Writing image file for inclusion: %s', filename)
             Image.fromarray(im).save(filename)
             oid = oid or 'Im_' + self._make_id('image', filename)
@@ -1048,18 +1020,7 @@ class RendererSVG(RendererBase):
         return char_id.replace("%20", "_")
 
     def _draw_text_as_path(self, gc, x, y, s, prop, angle, ismath, mtext=None):
-        """
-        Draw the text by converting them to paths using the textpath module.
-
-        Parameters
-        ----------
-        s : str
-            text to be converted
-        prop : `matplotlib.font_manager.FontProperties`
-            font property
-        ismath : bool
-            If True, use mathtext parser. If "TeX", use *usetex* mode.
-        """
+        # docstring inherited
         writer = self.writer
 
         writer.comment(s)
@@ -1094,7 +1055,7 @@ class RendererSVG(RendererBase):
             self._update_glyph_map_defs(glyph_map_new)
 
             for glyph_id, xposition, yposition, scale in glyph_info:
-                attrib = {'xlink:href': '#%s' % glyph_id}
+                attrib = {'xlink:href': f'#{glyph_id}'}
                 if xposition != 0.0:
                     attrib['x'] = _short_float_fmt(xposition)
                 if yposition != 0.0:
@@ -1119,7 +1080,7 @@ class RendererSVG(RendererBase):
                         ('translate', (xposition, yposition)),
                         ('scale', (scale,)),
                         ]),
-                    attrib={'xlink:href': '#%s' % char_id})
+                    attrib={'xlink:href': f'#{char_id}'})
 
             for verts, codes in rects:
                 path = Path(verts, codes)
@@ -1152,47 +1113,31 @@ class RendererSVG(RendererBase):
             if weight != 400:
                 font_parts.append(f'{weight}')
 
-            def _format_font_name(fn):
-                normalize_names = {
-                    'sans': 'sans-serif',
-                    'sans serif': 'sans-serif'
-                }
-                # A generic font family.  We need to do two things:
-                #  1. list all of the configured fonts with quoted names
-                #  2. append the generic name unquoted
+            def _normalize_sans(name):
+                return 'sans-serif' if name in ['sans', 'sans serif'] else name
+
+            def _expand_family_entry(fn):
+                fn = _normalize_sans(fn)
+                # prepend generic font families with all configured font names
                 if fn in fm.font_family_aliases:
-                    # fix spelling of sans-serif
-                    # we accept 3 ways CSS only supports 1
-                    fn = normalize_names.get(fn, fn)
                     # get all of the font names and fix spelling of sans-serif
-                    # if it comes back
-                    aliases = [
-                        normalize_names.get(_, _) for _ in
-                        fm.FontManager._expand_aliases(fn)
-                    ]
-                    # make sure the generic name appears at least once
-                    # duplicate is OK, next layer will deduplicate
-                    aliases.append(fn)
+                    # (we accept 3 ways CSS only supports 1)
+                    for name in fm.FontManager._expand_aliases(fn):
+                        yield _normalize_sans(name)
+                # whether a generic name or a family name, it must appear at
+                # least once
+                yield fn
 
-                    for a in aliases:
-                        # generic font families must not be quoted
-                        if a in fm.font_family_aliases:
-                            yield a
-                        # specific font families must be quoted
-                        else:
-                            yield repr(a)
-                # specific font families must be quoted
-                else:
-                    yield repr(fn)
-
-            def _get_all_names(prop):
-                for f in prop.get_family():
-                    yield from _format_font_name(f)
+            def _get_all_quoted_names(prop):
+                # only quote specific names, not generic names
+                return [name if name in fm.font_family_aliases else repr(name)
+                        for entry in prop.get_family()
+                        for name in _expand_family_entry(entry)]
 
             font_parts.extend([
                 f'{_short_float_fmt(prop.get_size())}px',
-                # ensure quoting and expansion of font names
-                ", ".join(dict.fromkeys(_get_all_names(prop)))
+                # ensure expansion, quoting, and dedupe of font names
+                ", ".join(dict.fromkeys(_get_all_quoted_names(prop)))
             ])
             style['font'] = ' '.join(font_parts)
             if prop.get_stretch() != 'normal':
@@ -1306,10 +1251,6 @@ class RendererSVG(RendererBase):
 
             writer.end('g')
 
-    def draw_tex(self, gc, x, y, s, prop, angle, *, mtext=None):
-        # docstring inherited
-        self._draw_text_as_path(gc, x, y, s, prop, angle, ismath="TeX")
-
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False, mtext=None):
         # docstring inherited
 
@@ -1352,9 +1293,7 @@ class FigureCanvasSVG(FigureCanvasBase):
 
     fixed_dpi = 72
 
-    @_api.delete_parameter("3.5", "args")
-    def print_svg(self, filename, *args, bbox_inches_restore=None,
-                  metadata=None):
+    def print_svg(self, filename, *, bbox_inches_restore=None, metadata=None):
         """
         Parameters
         ----------
@@ -1400,8 +1339,7 @@ class FigureCanvasSVG(FigureCanvasBase):
             self.figure.draw(renderer)
             renderer.finalize()
 
-    @_api.delete_parameter("3.5", "args")
-    def print_svgz(self, filename, *args, **kwargs):
+    def print_svgz(self, filename, **kwargs):
         with cbook.open_file_cm(filename, "wb") as fh, \
                 gzip.GzipFile(mode='w', fileobj=fh) as gzipwriter:
             return self.print_svg(gzipwriter, **kwargs)
@@ -1426,4 +1364,5 @@ svgProlog = """\
 
 @_Backend.export
 class _BackendSVG(_Backend):
+    backend_version = mpl.__version__
     FigureCanvas = FigureCanvasSVG
