@@ -107,7 +107,7 @@ def _get_packed_offsets(widths, total, sep, mode="fixed"):
         Widths of boxes to be packed.
     total : float or None
         Intended total length. *None* if not used.
-    sep : float
+    sep : float or None
         Spacing between boxes.
     mode : {'fixed', 'expand', 'equal'}
         The packing mode.
@@ -253,7 +253,7 @@ class OffsetBox(martist.Artist):
 
         Parameters
         ----------
-        mouseevent : `matplotlib.backend_bases.MouseEvent`
+        mouseevent : `~matplotlib.backend_bases.MouseEvent`
 
         Returns
         -------
@@ -268,9 +268,8 @@ class OffsetBox(martist.Artist):
         --------
         .Artist.contains
         """
-        inside, info = self._default_contains(mouseevent)
-        if inside is not None:
-            return inside, info
+        if self._different_canvas(mouseevent):
+            return False, {}
         for c in self.get_children():
             a, b = c.contains(mouseevent)
             if a:
@@ -533,8 +532,7 @@ class PaddedBox(OffsetBox):
     it when rendering.
     """
 
-    @_api.make_keyword_only("3.6", name="draw_frame")
-    def __init__(self, child, pad=0., draw_frame=False, patch_attrs=None):
+    def __init__(self, child, pad=0., *, draw_frame=False, patch_attrs=None):
         """
         Parameters
         ----------
@@ -715,8 +713,8 @@ class TextArea(OffsetBox):
     child text.
     """
 
-    @_api.make_keyword_only("3.6", name="textprops")
     def __init__(self, s,
+                 *,
                  textprops=None,
                  multilinebaseline=False,
                  ):
@@ -929,8 +927,7 @@ class AnchoredOffsetbox(OffsetBox):
              'center': 10,
              }
 
-    @_api.make_keyword_only("3.6", name="pad")
-    def __init__(self, loc,
+    def __init__(self, loc, *,
                  pad=0.4, borderpad=0.5,
                  child=None, prop=None, frameon=True,
                  bbox_to_anchor=None,
@@ -1103,8 +1100,7 @@ class AnchoredText(AnchoredOffsetbox):
     AnchoredOffsetbox with Text.
     """
 
-    @_api.make_keyword_only("3.6", name="pad")
-    def __init__(self, s, loc, pad=0.4, borderpad=0.5, prop=None, **kwargs):
+    def __init__(self, s, loc, *, pad=0.4, borderpad=0.5, prop=None, **kwargs):
         """
         Parameters
         ----------
@@ -1144,8 +1140,7 @@ class AnchoredText(AnchoredOffsetbox):
 
 class OffsetImage(OffsetBox):
 
-    @_api.make_keyword_only("3.6", name="zoom")
-    def __init__(self, arr,
+    def __init__(self, arr, *,
                  zoom=1,
                  cmap=None,
                  norm=None,
@@ -1226,14 +1221,10 @@ class AnnotationBbox(martist.Artist, mtext._AnnotationBase):
     zorder = 3
 
     def __str__(self):
-        return "AnnotationBbox(%g,%g)" % (self.xy[0], self.xy[1])
+        return f"AnnotationBbox({self.xy[0]:g},{self.xy[1]:g})"
 
     @_docstring.dedent_interpd
-    @_api.make_keyword_only("3.6", name="xycoords")
-    def __init__(self, offsetbox, xy,
-                 xybox=None,
-                 xycoords='data',
-                 boxcoords=None,
+    def __init__(self, offsetbox, xy, xybox=None, xycoords='data', boxcoords=None, *,
                  frameon=True, pad=0.4,  # FancyBboxPatch boxstyle.
                  annotation_clip=None,
                  box_alignment=(0.5, 0.5),
@@ -1359,9 +1350,8 @@ or callable, default: value of *xycoords*
         self.stale = True
 
     def contains(self, mouseevent):
-        inside, info = self._default_contains(mouseevent)
-        if inside is not None:
-            return inside, info
+        if self._different_canvas(mouseevent):
+            return False, {}
         if not self._check_xy(None):
             return False, {}
         return self.offsetbox.contains(mouseevent)
@@ -1399,28 +1389,22 @@ or callable, default: value of *xycoords*
         # docstring inherited
         if renderer is None:
             renderer = self.figure._get_renderer()
+        self.update_positions(renderer)
         return Bbox.union([child.get_window_extent(renderer)
                            for child in self.get_children()])
 
     def get_tightbbox(self, renderer=None):
         # docstring inherited
+        if renderer is None:
+            renderer = self.figure._get_renderer()
+        self.update_positions(renderer)
         return Bbox.union([child.get_tightbbox(renderer)
                            for child in self.get_children()])
 
     def update_positions(self, renderer):
-        """
-        Update pixel positions for the annotated point, the text and the arrow.
-        """
-
-        x, y = self.xybox
-        if isinstance(self.boxcoords, tuple):
-            xcoord, ycoord = self.boxcoords
-            x1, y1 = self._get_xy(renderer, x, y, xcoord)
-            x2, y2 = self._get_xy(renderer, x, y, ycoord)
-            ox0, oy0 = x1, y2
-        else:
-            ox0, oy0 = self._get_xy(renderer, x, y, self.boxcoords)
+        """Update pixel positions for the annotated point, the text, and the arrow."""
 
+        ox0, oy0 = self._get_xy(renderer, self.xybox, self.boxcoords)
         bbox = self.offsetbox.get_bbox(renderer)
         fw, fh = self._box_alignment
         self.offsetbox.set_offset(
@@ -1506,16 +1490,23 @@ class DraggableBase:
             ref_artist.set_picker(True)
         self.got_artist = False
         self._use_blit = use_blit and self.canvas.supports_blit
-        self.cids = [
-            self.canvas.callbacks._connect_picklable(
-                'pick_event', self.on_pick),
-            self.canvas.callbacks._connect_picklable(
-                'button_release_event', self.on_release),
+        callbacks = ref_artist.figure._canvas_callbacks
+        self._disconnectors = [
+            functools.partial(
+                callbacks.disconnect, callbacks._connect_picklable(name, func))
+            for name, func in [
+                ("pick_event", self.on_pick),
+                ("button_release_event", self.on_release),
+                ("motion_notify_event", self.on_motion),
+            ]
         ]
 
     # A property, not an attribute, to maintain picklability.
     canvas = property(lambda self: self.ref_artist.figure.canvas)
 
+    cids = property(lambda self: [
+        disconnect.args[0] for disconnect in self._disconnectors[:2]])
+
     def on_motion(self, evt):
         if self._check_still_parented() and self.got_artist:
             dx = evt.x - self.mouse_x
@@ -1542,16 +1533,12 @@ class DraggableBase:
                 self.ref_artist.draw(
                     self.ref_artist.figure._get_renderer())
                 self.canvas.blit()
-            self._c1 = self.canvas.callbacks._connect_picklable(
-                "motion_notify_event", self.on_motion)
             self.save_offset()
 
     def on_release(self, event):
         if self._check_still_parented() and self.got_artist:
             self.finalize_offset()
             self.got_artist = False
-            self.canvas.mpl_disconnect(self._c1)
-
             if self._use_blit:
                 self.ref_artist.set_animated(False)
 
@@ -1564,14 +1551,8 @@ class DraggableBase:
 
     def disconnect(self):
         """Disconnect the callbacks."""
-        for cid in self.cids:
-            self.canvas.mpl_disconnect(cid)
-        try:
-            c1 = self._c1
-        except AttributeError:
-            pass
-        else:
-            self.canvas.mpl_disconnect(c1)
+        for disconnector in self._disconnectors:
+            disconnector()
 
     def save_offset(self):
         pass
