@@ -62,7 +62,7 @@ def test_text_urls():
         fig.savefig(fd, format='svg')
         buf = fd.getvalue().decode()
 
-    expected = '<a xlink:href="{0}">'.format(test_url)
+    expected = f'<a xlink:href="{test_url}">'
     assert expected in buf
 
 
