@@ -1,4 +1,3 @@
-import copy
 import re
 
 import numpy as np
@@ -54,15 +53,32 @@ def test_path_exceptions():
 
 def test_point_in_path():
     # Test #1787
-    verts2 = [(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)]
-
-    path = Path(verts2, closed=True)
+    path = Path._create_closed([(0, 0), (0, 1), (1, 1), (1, 0)])
     points = [(0.5, 0.5), (1.5, 0.5)]
     ret = path.contains_points(points)
     assert ret.dtype == 'bool'
     np.testing.assert_equal(ret, [True, False])
 
 
+@pytest.mark.parametrize(
+    "other_path, inside, inverted_inside",
+    [(Path([(0.25, 0.25), (0.25, 0.75), (0.75, 0.75), (0.75, 0.25), (0.25, 0.25)],
+           closed=True), True, False),
+     (Path([(-0.25, -0.25), (-0.25, 1.75), (1.75, 1.75), (1.75, -0.25), (-0.25, -0.25)],
+           closed=True), False, True),
+     (Path([(-0.25, -0.25), (-0.25, 1.75), (0.5, 0.5),
+            (1.75, 1.75), (1.75, -0.25), (-0.25, -0.25)],
+           closed=True), False, False),
+     (Path([(0.25, 0.25), (0.25, 1.25), (1.25, 1.25), (1.25, 0.25), (0.25, 0.25)],
+           closed=True), False, False),
+     (Path([(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)], closed=True), False, False),
+     (Path([(2, 2), (2, 3), (3, 3), (3, 2), (2, 2)], closed=True), False, False)])
+def test_contains_path(other_path, inside, inverted_inside):
+    path = Path([(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)], closed=True)
+    assert path.contains_path(other_path) is inside
+    assert other_path.contains_path(path) is inverted_inside
+
+
 def test_contains_points_negative_radius():
     path = Path.unit_circle()
 
@@ -206,8 +222,14 @@ def test_log_transform_with_zero():
 def test_make_compound_path_empty():
     # We should be able to make a compound path with no arguments.
     # This makes it easier to write generic path based code.
-    r = Path.make_compound_path()
-    assert r.vertices.shape == (0, 2)
+    empty = Path.make_compound_path()
+    assert empty.vertices.shape == (0, 2)
+    r2 = Path.make_compound_path(empty, empty)
+    assert r2.vertices.shape == (0, 2)
+    assert r2.codes.shape == (0,)
+    r3 = Path.make_compound_path(Path([(0, 0)]), empty)
+    assert r3.vertices.shape == (1, 2)
+    assert r3.codes.shape == (1,)
 
 
 def test_make_compound_path_stops():
@@ -333,8 +355,28 @@ def test_path_deepcopy():
     codes = [Path.MOVETO, Path.LINETO]
     path1 = Path(verts)
     path2 = Path(verts, codes)
-    copy.deepcopy(path1)
-    copy.deepcopy(path2)
+    path1_copy = path1.deepcopy()
+    path2_copy = path2.deepcopy()
+    assert path1 is not path1_copy
+    assert path1.vertices is not path1_copy.vertices
+    assert path2 is not path2_copy
+    assert path2.vertices is not path2_copy.vertices
+    assert path2.codes is not path2_copy.codes
+
+
+def test_path_shallowcopy():
+    # Should not raise any error
+    verts = [[0, 0], [1, 1]]
+    codes = [Path.MOVETO, Path.LINETO]
+    path1 = Path(verts)
+    path2 = Path(verts, codes)
+    path1_copy = path1.copy()
+    path2_copy = path2.copy()
+    assert path1 is not path1_copy
+    assert path1.vertices is path1_copy.vertices
+    assert path2 is not path2_copy
+    assert path2.vertices is path2_copy.vertices
+    assert path2.codes is path2_copy.codes
 
 
 @pytest.mark.parametrize('phi', np.concatenate([
