@@ -3,9 +3,9 @@ import itertools
 import numpy as np
 import pytest
 
+from matplotlib.axes import Axes, SubplotBase
 import matplotlib.pyplot as plt
-from matplotlib.testing.decorators import image_comparison
-import matplotlib.axes as maxes
+from matplotlib.testing.decorators import check_figures_equal, image_comparison
 
 
 def check_shared(axs, x_shared, y_shared):
@@ -19,14 +19,13 @@ def check_shared(axs, x_shared, y_shared):
             enumerate(zip("xy", [x_shared, y_shared]))):
         if i2 <= i1:
             continue
-        assert \
-            (getattr(axs[0], "_shared_{}_axes".format(name)).joined(ax1, ax2)
-             == shared[i1, i2]), \
+        assert axs[0]._shared_axes[name].joined(ax1, ax2) == shared[i1, i2], \
             "axes %i and %i incorrectly %ssharing %s axis" % (
                 i1, i2, "not " if shared[i1, i2] else "", name)
 
 
-def check_visible(axs, x_visible, y_visible):
+def check_ticklabel_visible(axs, x_visible, y_visible):
+    """Check that the x and y ticklabel visibility is as specified."""
     for i, (ax, vx, vy) in enumerate(zip(axs, x_visible, y_visible)):
         for l in ax.get_xticklabels() + [ax.xaxis.offsetText]:
             assert l.get_visible() == vx, \
@@ -34,6 +33,26 @@ def check_visible(axs, x_visible, y_visible):
         for l in ax.get_yticklabels() + [ax.yaxis.offsetText]:
             assert l.get_visible() == vy, \
                     f"Visibility of y axis #{i} is incorrectly {vy}"
+        # axis label "visibility" is toggled by label_outer by resetting the
+        # label to empty, but it can also be empty to start with.
+        if not vx:
+            assert ax.get_xlabel() == ""
+        if not vy:
+            assert ax.get_ylabel() == ""
+
+
+def check_tick1_visible(axs, x_visible, y_visible):
+    """
+    Check that the x and y tick visibility is as specified.
+
+    Note: This only checks the tick1line, i.e. bottom / left ticks.
+    """
+    for ax, visible, in zip(axs, x_visible):
+        for tick in ax.xaxis.get_major_ticks():
+            assert tick.tick1line.get_visible() == visible
+    for ax, y_visible, in zip(axs, y_visible):
+        for tick in ax.yaxis.get_major_ticks():
+            assert tick.tick1line.get_visible() == visible
 
 
 def test_shared():
@@ -80,21 +99,30 @@ def test_shared():
     plt.close(f)
 
     # test all option combinations
-    ops = [False, True, 'all', 'none', 'row', 'col']
+    ops = [False, True, 'all', 'none', 'row', 'col', 0, 1]
     for xo in ops:
         for yo in ops:
             f, ((a1, a2), (a3, a4)) = plt.subplots(2, 2, sharex=xo, sharey=yo)
             axs = [a1, a2, a3, a4]
             check_shared(axs, share[xo], share[yo])
-            check_visible(axs, visible['x'][xo], visible['y'][yo])
+            check_ticklabel_visible(axs, visible['x'][xo], visible['y'][yo])
             plt.close(f)
 
-    # test label_outer
-    f, ((a1, a2), (a3, a4)) = plt.subplots(2, 2, sharex=True, sharey=True)
-    axs = [a1, a2, a3, a4]
-    for ax in axs:
-        ax.label_outer()
-    check_visible(axs, [False, False, True, True], [True, False, True, False])
+
+@pytest.mark.parametrize('remove_ticks', [True, False])
+def test_label_outer(remove_ticks):
+    f, axs = plt.subplots(2, 2, sharex=True, sharey=True)
+    for ax in axs.flat:
+        ax.set(xlabel="foo", ylabel="bar")
+        ax.label_outer(remove_inner_ticks=remove_ticks)
+    check_ticklabel_visible(
+        axs.flat, [False, False, True, True], [True, False, True, False])
+    if remove_ticks:
+        check_tick1_visible(
+            axs.flat, [False, False, True, True], [True, False, True, False])
+    else:
+        check_tick1_visible(
+            axs.flat, [True, True, True, True], [True, True, True, True])
 
 
 def test_label_outer_span():
@@ -113,22 +141,28 @@ def test_label_outer_span():
     a4 = fig.add_subplot(gs[2, 1])
     for ax in fig.axes:
         ax.label_outer()
-    check_visible(
+    check_ticklabel_visible(
         fig.axes, [False, True, False, True], [True, True, False, False])
 
 
+def test_label_outer_non_gridspec():
+    ax = plt.axes((0, 0, 1, 1))
+    ax.label_outer()  # Does nothing.
+    check_ticklabel_visible([ax], [True], [True])
+
+
 def test_shared_and_moved():
     # test if sharey is on, but then tick_left is called that labels don't
     # re-appear.  Seaborn does this just to be sure yaxis is on left...
     f, (a1, a2) = plt.subplots(1, 2, sharey=True)
-    check_visible([a2], [True], [False])
+    check_ticklabel_visible([a2], [True], [False])
     a2.yaxis.tick_left()
-    check_visible([a2], [True], [False])
+    check_ticklabel_visible([a2], [True], [False])
 
     f, (a1, a2) = plt.subplots(2, 1, sharex=True)
-    check_visible([a1], [False], [True])
+    check_ticklabel_visible([a1], [False], [True])
     a2.xaxis.tick_bottom()
-    check_visible([a1], [False], [True])
+    check_ticklabel_visible([a1], [False], [True])
 
 
 def test_exceptions():
@@ -137,20 +171,9 @@ def test_exceptions():
         plt.subplots(2, 2, sharex='blah')
     with pytest.raises(ValueError):
         plt.subplots(2, 2, sharey='blah')
-    # We filter warnings in this test which are genuine since
-    # the point of this test is to ensure that this raises.
-    with pytest.warns(UserWarning, match='.*sharex argument to subplots'), \
-         pytest.raises(ValueError):
-        plt.subplots(2, 2, -1)
-    with pytest.warns(UserWarning, match='.*sharex argument to subplots'), \
-         pytest.raises(ValueError):
-        plt.subplots(2, 2, 0)
-    with pytest.warns(UserWarning, match='.*sharex argument to subplots'), \
-         pytest.raises(ValueError):
-        plt.subplots(2, 2, 5)
 
 
-@image_comparison(['subplots_offset_text'], remove_text=False)
+@image_comparison(['subplots_offset_text'])
 def test_subplots_offsettext():
     x = np.arange(0, 1e10, 1e9)
     y = np.arange(0, 100, 10)+1e4
@@ -165,7 +188,7 @@ def test_subplots_offsettext():
 @pytest.mark.parametrize("bottom", [True, False])
 @pytest.mark.parametrize("left", [True, False])
 @pytest.mark.parametrize("right", [True, False])
-def test_subplots_hide_labels(top, bottom, left, right):
+def test_subplots_hide_ticklabels(top, bottom, left, right):
     # Ideally, we would also test offset-text visibility (and remove
     # test_subplots_offsettext), but currently, setting rcParams fails to move
     # the offset texts as well.
@@ -183,6 +206,23 @@ def test_subplots_hide_labels(top, bottom, left, right):
         assert yright == (right and j == 2)
 
 
+@pytest.mark.parametrize("xlabel_position", ["bottom", "top"])
+@pytest.mark.parametrize("ylabel_position", ["left", "right"])
+def test_subplots_hide_axislabels(xlabel_position, ylabel_position):
+    axs = plt.figure().subplots(3, 3, sharex=True, sharey=True)
+    for (i, j), ax in np.ndenumerate(axs):
+        ax.set(xlabel="foo", ylabel="bar")
+        ax.xaxis.set_label_position(xlabel_position)
+        ax.yaxis.set_label_position(ylabel_position)
+        ax.label_outer()
+        assert bool(ax.get_xlabel()) == (
+            xlabel_position == "bottom" and i == 2
+            or xlabel_position == "top" and i == 0)
+        assert bool(ax.get_ylabel()) == (
+            ylabel_position == "left" and j == 0
+            or ylabel_position == "right" and j == 2)
+
+
 def test_get_gridspec():
     # ahem, pretty trivial, but...
     fig, ax = plt.subplots()
@@ -198,6 +238,48 @@ def test_dont_mutate_kwargs():
     assert gridspec_kw == {'width_ratios': [1, 2]}
 
 
-def test_subplot_factory_reapplication():
-    assert maxes.subplot_class_factory(maxes.Axes) is maxes.Subplot
-    assert maxes.subplot_class_factory(maxes.Subplot) is maxes.Subplot
+@pytest.mark.parametrize("width_ratios", [None, [1, 3, 2]])
+@pytest.mark.parametrize("height_ratios", [None, [1, 2]])
+@check_figures_equal(extensions=['png'])
+def test_width_and_height_ratios(fig_test, fig_ref,
+                                 height_ratios, width_ratios):
+    fig_test.subplots(2, 3, height_ratios=height_ratios,
+                      width_ratios=width_ratios)
+    fig_ref.subplots(2, 3, gridspec_kw={
+                     'height_ratios': height_ratios,
+                     'width_ratios': width_ratios})
+
+
+@pytest.mark.parametrize("width_ratios", [None, [1, 3, 2]])
+@pytest.mark.parametrize("height_ratios", [None, [1, 2]])
+@check_figures_equal(extensions=['png'])
+def test_width_and_height_ratios_mosaic(fig_test, fig_ref,
+                                        height_ratios, width_ratios):
+    mosaic_spec = [['A', 'B', 'B'], ['A', 'C', 'D']]
+    fig_test.subplot_mosaic(mosaic_spec, height_ratios=height_ratios,
+                            width_ratios=width_ratios)
+    fig_ref.subplot_mosaic(mosaic_spec, gridspec_kw={
+                           'height_ratios': height_ratios,
+                           'width_ratios': width_ratios})
+
+
+@pytest.mark.parametrize('method,args', [
+    ('subplots', (2, 3)),
+    ('subplot_mosaic', ('abc;def', ))
+    ]
+)
+def test_ratio_overlapping_kws(method, args):
+    with pytest.raises(ValueError, match='height_ratios'):
+        getattr(plt, method)(*args, height_ratios=[1, 2],
+                             gridspec_kw={'height_ratios': [1, 2]})
+    with pytest.raises(ValueError, match='width_ratios'):
+        getattr(plt, method)(*args, width_ratios=[1, 2, 3],
+                             gridspec_kw={'width_ratios': [1, 2, 3]})
+
+
+def test_old_subplot_compat():
+    fig = plt.figure()
+    assert isinstance(fig.add_subplot(), SubplotBase)
+    assert not isinstance(fig.add_axes(rect=[0, 0, 1, 1]), SubplotBase)
+    with pytest.raises(TypeError):
+        Axes(fig, [0, 0, 1, 1], rect=[0, 0, 1, 1])
