@@ -1,12 +1,14 @@
 from itertools import product
+import io
 import platform
 
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import matplotlib.ticker as mticker
-from matplotlib import _api, cbook
+from matplotlib import cbook
 from matplotlib.backend_bases import MouseEvent
 from matplotlib.colors import LogNorm
+from matplotlib.patches import Circle, Ellipse
 from matplotlib.transforms import Bbox, TransformedBbox
 from matplotlib.testing.decorators import (
     check_figures_equal, image_comparison, remove_ticks_and_titles)
@@ -16,7 +18,8 @@ from mpl_toolkits.axes_grid1 import (
     host_subplot, make_axes_locatable,
     Grid, AxesGrid, ImageGrid)
 from mpl_toolkits.axes_grid1.anchored_artists import (
-    AnchoredSizeBar, AnchoredDirectionArrows)
+    AnchoredAuxTransformBox, AnchoredDrawingArea, AnchoredEllipse,
+    AnchoredDirectionArrows, AnchoredSizeBar)
 from mpl_toolkits.axes_grid1.axes_divider import (
     Divider, HBoxDivider, make_axes_area_auto_adjustable, SubplotDivider,
     VBoxDivider)
@@ -25,7 +28,6 @@ from mpl_toolkits.axes_grid1.inset_locator import (
     zoomed_inset_axes, mark_inset, inset_axes, BboxConnectorPatch,
     InsetPosition)
 import mpl_toolkits.axes_grid1.mpl_axes
-
 import pytest
 
 import numpy as np
@@ -89,6 +91,15 @@ def test_twin_axes_empty_and_removed():
     plt.subplots_adjust(wspace=0.5, hspace=1)
 
 
+def test_twin_axes_both_with_units():
+    host = host_subplot(111)
+    host.plot_date([0, 1, 2], [0, 1, 2], xdate=False, ydate=True)
+    twin = host.twinx()
+    twin.plot(["a", "b", "c"])
+    assert host.get_yticklabels()[0].get_text() == "00:00:00"
+    assert twin.get_yticklabels()[0].get_text() == "a"
+
+
 def test_axesgrid_colorbar_log_smoketest():
     fig = plt.figure()
     grid = AxesGrid(fig, 111,  # modified to be only subplot
@@ -123,7 +134,7 @@ def test_inset_locator():
 
     # prepare the demo image
     # Z is a 15x15 array
-    Z = cbook.get_sample_data("axes_grid/bivariate_normal.npy", np_load=True)
+    Z = cbook.get_sample_data("axes_grid/bivariate_normal.npy")
     extent = (-3, 4, -4, 3)
     Z2 = np.zeros((150, 150))
     ny, nx = Z.shape
@@ -164,7 +175,7 @@ def test_inset_axes():
 
     # prepare the demo image
     # Z is a 15x15 array
-    Z = cbook.get_sample_data("axes_grid/bivariate_normal.npy", np_load=True)
+    Z = cbook.get_sample_data("axes_grid/bivariate_normal.npy")
     extent = (-3, 4, -4, 3)
     Z2 = np.zeros((150, 150))
     ny, nx = Z.shape
@@ -245,6 +256,15 @@ def test_inset_axes_complete():
                          bbox_transform=ax.transAxes)
 
 
+def test_inset_axes_tight():
+    # gh-26287 found that inset_axes raised with bbox_inches=tight
+    fig, ax = plt.subplots()
+    inset_axes(ax, width=1.3, height=0.9)
+
+    f = io.BytesIO()
+    fig.savefig(f, bbox_inches="tight")
+
+
 @image_comparison(['fill_facecolor.png'], remove_text=True, style='mpl20')
 def test_fill_facecolor():
     fig, ax = plt.subplots(1, 5)
@@ -415,7 +435,7 @@ def test_image_grid_label_mode_deprecation_warning():
     imdata = np.arange(9).reshape((3, 3))
 
     fig = plt.figure()
-    with pytest.warns(_api.MatplotlibDeprecationWarning,
+    with pytest.warns(mpl.MatplotlibDeprecationWarning,
                       match="Passing an undefined label_mode"):
         grid = ImageGrid(fig, (0, 0, 1, 1), (2, 1), label_mode="foo")
 
@@ -508,6 +528,36 @@ def test_picking_callbacks_overlap(big_on_axes, small_on_axes, click_on):
         assert small in event_rects
 
 
+@image_comparison(['anchored_artists.png'], remove_text=True, style='mpl20')
+def test_anchored_artists():
+    fig, ax = plt.subplots(figsize=(3, 3))
+    ada = AnchoredDrawingArea(40, 20, 0, 0, loc='upper right', pad=0.,
+                              frameon=False)
+    p1 = Circle((10, 10), 10)
+    ada.drawing_area.add_artist(p1)
+    p2 = Circle((30, 10), 5, fc="r")
+    ada.drawing_area.add_artist(p2)
+    ax.add_artist(ada)
+
+    box = AnchoredAuxTransformBox(ax.transData, loc='upper left')
+    el = Ellipse((0, 0), width=0.1, height=0.4, angle=30, color='cyan')
+    box.drawing_area.add_artist(el)
+    ax.add_artist(box)
+
+    # Manually construct the ellipse instead, once the deprecation elapses.
+    with pytest.warns(mpl.MatplotlibDeprecationWarning):
+        ae = AnchoredEllipse(ax.transData, width=0.1, height=0.25, angle=-60,
+                             loc='lower left', pad=0.5, borderpad=0.4,
+                             frameon=True)
+    ax.add_artist(ae)
+
+    asb = AnchoredSizeBar(ax.transData, 0.2, r"0.2 units", loc='lower right',
+                          pad=0.3, borderpad=0.4, sep=4, fill_bar=True,
+                          frameon=False, label_top=True, prop={'size': 20},
+                          size_vertical=0.05, color='green')
+    ax.add_artist(asb)
+
+
 def test_hbox_divider():
     arr1 = np.arange(20).reshape((4, 5))
     arr2 = np.arange(20).reshape((5, 4))
@@ -578,9 +628,14 @@ def test_grid_axes_position(direction):
     fig = plt.figure()
     grid = Grid(fig, 111, (2, 2), direction=direction)
     loc = [ax.get_axes_locator() for ax in np.ravel(grid.axes_row)]
-    assert loc[1]._nx > loc[0]._nx and loc[2]._ny < loc[0]._ny
-    assert loc[0]._nx == loc[2]._nx and loc[0]._ny == loc[1]._ny
-    assert loc[3]._nx == loc[1]._nx and loc[3]._ny == loc[2]._ny
+    # Test nx.
+    assert loc[1].args[0] > loc[0].args[0]
+    assert loc[0].args[0] == loc[2].args[0]
+    assert loc[3].args[0] == loc[1].args[0]
+    # Test ny.
+    assert loc[2].args[1] < loc[0].args[1]
+    assert loc[0].args[1] == loc[1].args[1]
+    assert loc[3].args[1] == loc[2].args[1]
 
 
 @pytest.mark.parametrize('rect, ngrids, error, message', (
@@ -654,7 +709,8 @@ def test_rgb_axes():
 def test_insetposition():
     fig, ax = plt.subplots(figsize=(2, 2))
     ax_ins = plt.axes([0, 0, 1, 1])
-    ip = InsetPosition(ax, [0.2, 0.25, 0.5, 0.4])
+    with pytest.warns(mpl.MatplotlibDeprecationWarning):
+        ip = InsetPosition(ax, [0.2, 0.25, 0.5, 0.4])
     ax_ins.set_axes_locator(ip)
 
 
@@ -664,11 +720,7 @@ def test_insetposition():
 @image_comparison(['imagegrid_cbar_mode.png'],
                   remove_text=True, style='mpl20', tol=0.3)
 def test_imagegrid_cbar_mode_edge():
-    # Remove this line when this test image is regenerated.
-    plt.rcParams['pcolormesh.snap'] = False
-
-    X, Y = np.meshgrid(np.linspace(0, 6, 30), np.linspace(0, 6, 30))
-    arr = np.sin(X) * np.cos(Y) + 1j*(np.sin(3*Y) * np.cos(Y/2.))
+    arr = np.arange(16).reshape((4, 4))
 
     fig = plt.figure(figsize=(18, 9))
 
@@ -684,12 +736,12 @@ def test_imagegrid_cbar_mode_edge():
                          cbar_location=location,
                          cbar_size='20%',
                          cbar_mode='edge')
-        ax1, ax2, ax3, ax4, = grid
+        ax1, ax2, ax3, ax4 = grid
 
-        ax1.imshow(arr.real, cmap='nipy_spectral')
-        ax2.imshow(arr.imag, cmap='hot')
-        ax3.imshow(np.abs(arr), cmap='jet')
-        ax4.imshow(np.arctan2(arr.imag, arr.real), cmap='hsv')
+        ax1.imshow(arr, cmap='nipy_spectral')
+        ax2.imshow(arr.T, cmap='hot')
+        ax3.imshow(np.hypot(arr, arr.T), cmap='jet')
+        ax4.imshow(np.arctan2(arr, arr.T), cmap='hsv')
 
         # In each row/column, the "first" colorbars must be overwritten by the
         # "second" ones.  To achieve this, clear out the axes first.
@@ -727,12 +779,15 @@ def test_anchored_locator_base_call():
     ax.set(aspect=1, xlim=(-15, 15), ylim=(-20, 5))
     ax.set(xticks=[], yticks=[])
 
-    Z = cbook.get_sample_data(
-        "axes_grid/bivariate_normal.npy", np_load=True
-    )
+    Z = cbook.get_sample_data("axes_grid/bivariate_normal.npy")
     extent = (-3, 4, -4, 3)
 
     axins = zoomed_inset_axes(ax, zoom=2, loc="upper left")
     axins.set(xticks=[], yticks=[])
 
     axins.imshow(Z, extent=extent, origin="lower")
+
+
+def test_grid_with_axes_class_not_overriding_axis():
+    Grid(plt.figure(), 111, (2, 2), axes_class=mpl.axes.Axes)
+    RGBAxes(plt.figure(), 111, axes_class=mpl.axes.Axes)
