@@ -1,13 +1,19 @@
-from matplotlib.patches import _Style, FancyArrowPatch
-from matplotlib.transforms import IdentityTransform
-from matplotlib.path import Path
+"""
+Provides classes to style the axis lines.
+"""
+import math
+
 import numpy as np
 
-class _FancyAxislineStyle(object):
+import matplotlib as mpl
+from matplotlib.patches import _Style, FancyArrowPatch
+from matplotlib.path import Path
+from matplotlib.transforms import IdentityTransform
+
+
+class _FancyAxislineStyle:
     class SimpleArrow(FancyArrowPatch):
-        """
-        The artist class that will be returned for SimpleArrow style.
-        """
+        """The artist class that will be returned for SimpleArrow style."""
         _ARROW_STYLE = "->"
 
         def __init__(self, axis_artist, line_path, transform,
@@ -20,7 +26,6 @@ class _FancyAxislineStyle(object):
             FancyArrowPatch.__init__(self,
                                      path=self._line_path,
                                      arrowstyle=self._ARROW_STYLE,
-                                     arrow_transmuter=None,
                                      patchA=None,
                                      patchB=None,
                                      shrinkA=0.,
@@ -37,22 +42,15 @@ class _FancyAxislineStyle(object):
             """
             Extend the path to make a room for drawing arrow.
             """
-            from matplotlib.bezier import get_cos_sin
-
-            x0, y0 = path.vertices[-2]
-            x1, y1 = path.vertices[-1]
-            cost, sint = get_cos_sin(x0, y0, x1, y1)
-
-            d = mutation_size * 1.
-            x2, y2 = x1 + cost*d, y1+sint*d
-
+            (x0, y0), (x1, y1) = path.vertices[-2:]
+            theta = math.atan2(y1 - y0, x1 - x0)
+            x2 = x1 + math.cos(theta) * mutation_size
+            y2 = y1 + math.sin(theta) * mutation_size
             if path.codes is None:
-                _path = Path(np.concatenate([path.vertices, [[x2, y2]]]))
+                return Path(np.concatenate([path.vertices, [[x2, y2]]]))
             else:
-                _path = Path(np.concatenate([path.vertices, [[x2, y2]]]),
-                             np.concatenate([path.codes, [Path.LINETO]]))
-
-            return _path
+                return Path(np.concatenate([path.vertices, [[x2, y2]]]),
+                            np.concatenate([path.codes, [Path.LINETO]]))
 
         def set_path(self, path):
             self._line_path = path
@@ -60,38 +58,48 @@ class _FancyAxislineStyle(object):
         def draw(self, renderer):
             """
             Draw the axis line.
-             1) transform the path to the display coordinate.
-             2) extend the path to make a room for arrow
-             3) update the path of the FancyArrowPatch.
-             4) draw
+             1) Transform the path to the display coordinate.
+             2) Extend the path to make a room for arrow.
+             3) Update the path of the FancyArrowPatch.
+             4) Draw.
             """
             path_in_disp = self._line_transform.transform_path(self._line_path)
-            mutation_size = self.get_mutation_scale() #line_mutation_scale()
-            extented_path = self._extend_path(path_in_disp,
+            mutation_size = self.get_mutation_scale()  # line_mutation_scale()
+            extended_path = self._extend_path(path_in_disp,
                                               mutation_size=mutation_size)
-
-            self._path_original = extented_path
+            self._path_original = extended_path
             FancyArrowPatch.draw(self, renderer)
 
+        def get_window_extent(self, renderer=None):
+
+            path_in_disp = self._line_transform.transform_path(self._line_path)
+            mutation_size = self.get_mutation_scale()  # line_mutation_scale()
+            extended_path = self._extend_path(path_in_disp,
+                                              mutation_size=mutation_size)
+            self._path_original = extended_path
+            return FancyArrowPatch.get_window_extent(self, renderer)
+
     class FilledArrow(SimpleArrow):
-        """
-        The artist class that will be returned for SimpleArrow style.
-        """
+        """The artist class that will be returned for FilledArrow style."""
         _ARROW_STYLE = "-|>"
 
+        def __init__(self, axis_artist, line_path, transform,
+                     line_mutation_scale, facecolor):
+            super().__init__(axis_artist, line_path, transform,
+                             line_mutation_scale)
+            self.set_facecolor(facecolor)
+
 
 class AxislineStyle(_Style):
     """
-    :class:`AxislineStyle` is a container class which defines style classes
-    for AxisArtists.
+    A container class which defines style classes for AxisArtists.
 
-    An instance of any axisline style class is an callable object,
+    An instance of any axisline style class is a callable object,
     whose call signature is ::
 
        __call__(self, axis_artist, path, transform)
 
-    When called, this should return a mpl artist with following
-    methods implemented. ::
+    When called, this should return an `.Artist` with the following methods::
 
       def set_path(self, path):
           # set the path for axisline.
@@ -101,14 +109,11 @@ class AxislineStyle(_Style):
 
       def draw(self, renderer):
           # draw
-
-
     """
 
     _style_list = {}
 
-
-    class _Base(object):
+    class _Base:
         # The derived classes are required to be able to be initialized
         # w/o arguments, i.e., all its argument (except self) must have
         # the default values.
@@ -119,18 +124,13 @@ class AxislineStyle(_Style):
             """
             super().__init__()
 
-
-
-
         def __call__(self, axis_artist, transform):
             """
-            Given the AxisArtist instance, and transform for the path
-            (set_path method), return the mpl artist for drawing the axis line.
+            Given the AxisArtist instance, and transform for the path (set_path
+            method), return the Matplotlib artist for drawing the axis line.
             """
-
             return self.new_line(axis_artist, transform)
 
-
     class SimpleArrow(_Base):
         """
         A simple arrow.
@@ -140,8 +140,10 @@ class AxislineStyle(_Style):
 
         def __init__(self, size=1):
             """
-             *size*
-                size of the arrow as a fraction of the ticklabel size.
+            Parameters
+            ----------
+            size : float
+                Size of the arrow as a fraction of the ticklabel size.
             """
 
             self.size = size
@@ -149,15 +151,43 @@ class AxislineStyle(_Style):
 
         def new_line(self, axis_artist, transform):
 
-            linepath = Path([(0,0), (0, 1)])
+            linepath = Path([(0, 0), (0, 1)])
             axisline = self.ArrowAxisClass(axis_artist, linepath, transform,
                                            line_mutation_scale=self.size)
             return axisline
 
-
     _style_list["->"] = SimpleArrow
 
     class FilledArrow(SimpleArrow):
+        """
+        An arrow with a filled head.
+        """
+
         ArrowAxisClass = _FancyAxislineStyle.FilledArrow
 
+        def __init__(self, size=1, facecolor=None):
+            """
+            Parameters
+            ----------
+            size : float
+                Size of the arrow as a fraction of the ticklabel size.
+            facecolor : color, default: :rc:`axes.edgecolor`
+                Fill color.
+
+                .. versionadded:: 3.7
+            """
+
+            if facecolor is None:
+                facecolor = mpl.rcParams['axes.edgecolor']
+            self.size = size
+            self._facecolor = facecolor
+            super().__init__(size=size)
+
+        def new_line(self, axis_artist, transform):
+            linepath = Path([(0, 0), (0, 1)])
+            axisline = self.ArrowAxisClass(axis_artist, linepath, transform,
+                                           line_mutation_scale=self.size,
+                                           facecolor=self._facecolor)
+            return axisline
+
     _style_list["-|>"] = FilledArrow
