@@ -1,5 +1,6 @@
 import functools
 import itertools
+import platform
 
 import pytest
 
@@ -39,7 +40,37 @@ def test_invisible_axes(fig_test, fig_ref):
     ax.set_visible(False)
 
 
-@mpl3d_image_comparison(['aspects.png'], remove_text=False)
+@mpl3d_image_comparison(['grid_off.png'], style='mpl20')
+def test_grid_off():
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='3d')
+    ax.grid(False)
+
+
+@mpl3d_image_comparison(['invisible_ticks_axis.png'], style='mpl20')
+def test_invisible_ticks_axis():
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='3d')
+    ax.set_xticks([])
+    ax.set_yticks([])
+    ax.set_zticks([])
+    for axis in [ax.xaxis, ax.yaxis, ax.zaxis]:
+        axis.line.set_visible(False)
+
+
+@mpl3d_image_comparison(['axis_positions.png'], remove_text=False, style='mpl20')
+def test_axis_positions():
+    positions = ['upper', 'lower', 'both', 'none']
+    fig, axs = plt.subplots(2, 2, subplot_kw={'projection': '3d'})
+    for ax, pos in zip(axs.flatten(), positions):
+        for axis in ax.xaxis, ax.yaxis, ax.zaxis:
+            axis.set_label_position(pos)
+            axis.set_ticks_position(pos)
+        title = f'{pos}'
+        ax.set(xlabel='x', ylabel='y', zlabel='z', title=title)
+
+
+@mpl3d_image_comparison(['aspects.png'], remove_text=False, style='mpl20')
 def test_aspects():
     aspects = ('auto', 'equal', 'equalxy', 'equalyz', 'equalxz', 'equal')
     _, axs = plt.subplots(2, 3, subplot_kw={'projection': '3d'})
@@ -56,7 +87,8 @@ def test_aspects():
     axs[1][2].set_title('equal (cube)')
 
 
-@mpl3d_image_comparison(['aspects_adjust_box.png'], remove_text=False)
+@mpl3d_image_comparison(['aspects_adjust_box.png'],
+                        remove_text=False, style='mpl20')
 def test_aspects_adjust_box():
     aspects = ('auto', 'equal', 'equalxy', 'equalyz', 'equalxz')
     fig, axs = plt.subplots(1, len(aspects), subplot_kw={'projection': '3d'},
@@ -81,7 +113,7 @@ def test_axes3d_repr():
         "title={'center': 'title'}, xlabel='x', ylabel='y', zlabel='z'>")
 
 
-@mpl3d_image_comparison(['axes3d_primary_views.png'])
+@mpl3d_image_comparison(['axes3d_primary_views.png'], style='mpl20')
 def test_axes3d_primary_views():
     # (elev, azim, roll)
     views = [(90, -90, 0),  # XY
@@ -102,7 +134,7 @@ def test_axes3d_primary_views():
     plt.tight_layout()
 
 
-@mpl3d_image_comparison(['bar3d.png'])
+@mpl3d_image_comparison(['bar3d.png'], style='mpl20')
 def test_bar3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -126,7 +158,7 @@ def test_bar3d_colors():
         ax.bar3d(xs, ys, zs, 1, 1, 1, color=c)
 
 
-@mpl3d_image_comparison(['bar3d_shaded.png'])
+@mpl3d_image_comparison(['bar3d_shaded.png'], style='mpl20')
 def test_bar3d_shaded():
     x = np.arange(4)
     y = np.arange(5)
@@ -146,7 +178,7 @@ def test_bar3d_shaded():
     fig.canvas.draw()
 
 
-@mpl3d_image_comparison(['bar3d_notshaded.png'])
+@mpl3d_image_comparison(['bar3d_notshaded.png'], style='mpl20')
 def test_bar3d_notshaded():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -186,7 +218,9 @@ def test_bar3d_lightsource():
     np.testing.assert_array_max_ulp(color, collection._facecolor3d[1::6], 4)
 
 
-@mpl3d_image_comparison(['contour3d.png'])
+@mpl3d_image_comparison(
+    ['contour3d.png'], style='mpl20',
+    tol=0.002 if platform.machine() in ('aarch64', 'ppc64le', 's390x') else 0)
 def test_contour3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -194,12 +228,10 @@ def test_contour3d():
     ax.contour(X, Y, Z, zdir='z', offset=-100, cmap=cm.coolwarm)
     ax.contour(X, Y, Z, zdir='x', offset=-40, cmap=cm.coolwarm)
     ax.contour(X, Y, Z, zdir='y', offset=40, cmap=cm.coolwarm)
-    ax.set_xlim(-40, 40)
-    ax.set_ylim(-40, 40)
-    ax.set_zlim(-100, 100)
+    ax.axis(xmin=-40, xmax=40, ymin=-40, ymax=40, zmin=-100, zmax=100)
 
 
-@mpl3d_image_comparison(['contour3d_extend3d.png'])
+@mpl3d_image_comparison(['contour3d_extend3d.png'], style='mpl20')
 def test_contour3d_extend3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -210,7 +242,7 @@ def test_contour3d_extend3d():
     ax.set_zlim(-80, 80)
 
 
-@mpl3d_image_comparison(['contourf3d.png'])
+@mpl3d_image_comparison(['contourf3d.png'], style='mpl20')
 def test_contourf3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -223,7 +255,7 @@ def test_contourf3d():
     ax.set_zlim(-100, 100)
 
 
-@mpl3d_image_comparison(['contourf3d_fill.png'])
+@mpl3d_image_comparison(['contourf3d_fill.png'], style='mpl20')
 def test_contourf3d_fill():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -266,7 +298,7 @@ def test_contourf3d_extend(fig_test, fig_ref, extend, levels):
         ax.set_zlim(-10, 10)
 
 
-@mpl3d_image_comparison(['tricontour.png'], tol=0.02)
+@mpl3d_image_comparison(['tricontour.png'], tol=0.02, style='mpl20')
 def test_tricontour():
     fig = plt.figure()
 
@@ -292,7 +324,7 @@ def test_contour3d_1d_input():
     ax.contour(x, y, z, [0.5])
 
 
-@mpl3d_image_comparison(['lines3d.png'])
+@mpl3d_image_comparison(['lines3d.png'], style='mpl20')
 def test_lines3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -329,7 +361,7 @@ def test_invalid_line_data():
         line.set_data_3d([], [], 0)
 
 
-@mpl3d_image_comparison(['mixedsubplot.png'])
+@mpl3d_image_comparison(['mixedsubplot.png'], style='mpl20')
 def test_mixedsubplots():
     def f(t):
         return np.cos(2*np.pi*t) * np.exp(-t)
@@ -366,7 +398,7 @@ def test_tight_layout_text(fig_test, fig_ref):
     ax2.text(.5, .5, .5, s='some string')
 
 
-@mpl3d_image_comparison(['scatter3d.png'])
+@mpl3d_image_comparison(['scatter3d.png'], style='mpl20')
 def test_scatter3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -379,7 +411,7 @@ def test_scatter3d():
     ax.scatter([], [], [], c='r', marker='X')
 
 
-@mpl3d_image_comparison(['scatter3d_color.png'])
+@mpl3d_image_comparison(['scatter3d_color.png'], style='mpl20')
 def test_scatter3d_color():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -395,7 +427,7 @@ def test_scatter3d_color():
                color='b', marker='s')
 
 
-@mpl3d_image_comparison(['scatter3d_linewidth.png'])
+@mpl3d_image_comparison(['scatter3d_linewidth.png'], style='mpl20')
 def test_scatter3d_linewidth():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -541,7 +573,7 @@ def test_marker_draw_order_view_rotated(fig_test, fig_ref):
     ax.view_init(elev=0, azim=azim - 180, roll=0)  # view rotated by 180 deg
 
 
-@mpl3d_image_comparison(['plot_3d_from_2d.png'], tol=0.015)
+@mpl3d_image_comparison(['plot_3d_from_2d.png'], tol=0.015, style='mpl20')
 def test_plot_3d_from_2d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -551,7 +583,7 @@ def test_plot_3d_from_2d():
     ax.plot(xs, ys, zs=0, zdir='y')
 
 
-@mpl3d_image_comparison(['surface3d.png'])
+@mpl3d_image_comparison(['surface3d.png'], style='mpl20')
 def test_surface3d():
     # Remove this line when this test image is regenerated.
     plt.rcParams['pcolormesh.snap'] = False
@@ -569,7 +601,22 @@ def test_surface3d():
     fig.colorbar(surf, shrink=0.5, aspect=5)
 
 
-@mpl3d_image_comparison(['surface3d_shaded.png'])
+@image_comparison(['surface3d_label_offset_tick_position.png'], style='mpl20')
+def test_surface3d_label_offset_tick_position():
+    ax = plt.figure().add_subplot(projection="3d")
+
+    x, y = np.mgrid[0:6 * np.pi:0.25, 0:4 * np.pi:0.25]
+    z = np.sqrt(np.abs(np.cos(x) + np.cos(y)))
+
+    ax.plot_surface(x * 1e5, y * 1e6, z * 1e8, cmap='autumn', cstride=2, rstride=2)
+    ax.set_xlabel("X label")
+    ax.set_ylabel("Y label")
+    ax.set_zlabel("Z label")
+
+    ax.figure.canvas.draw()
+
+
+@mpl3d_image_comparison(['surface3d_shaded.png'], style='mpl20')
 def test_surface3d_shaded():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -583,7 +630,7 @@ def test_surface3d_shaded():
     ax.set_zlim(-1.01, 1.01)
 
 
-@mpl3d_image_comparison(['surface3d_masked.png'])
+@mpl3d_image_comparison(['surface3d_masked.png'], style='mpl20')
 def test_surface3d_masked():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -620,7 +667,7 @@ def test_plot_surface_None_arg(fig_test, fig_ref):
     ax_ref.plot_surface(x, y, z)
 
 
-@mpl3d_image_comparison(['surface3d_masked_strides.png'])
+@mpl3d_image_comparison(['surface3d_masked_strides.png'], style='mpl20')
 def test_surface3d_masked_strides():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -632,7 +679,7 @@ def test_surface3d_masked_strides():
     ax.view_init(60, -45, 0)
 
 
-@mpl3d_image_comparison(['text3d.png'], remove_text=False)
+@mpl3d_image_comparison(['text3d.png'], remove_text=False, style='mpl20')
 def test_text3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -681,7 +728,7 @@ def test_text3d_modification(fig_ref, fig_test):
         ax_ref.text(x, y, z, f'({x}, {y}, {z}), dir={zdir}', zdir=zdir)
 
 
-@mpl3d_image_comparison(['trisurf3d.png'], tol=0.061)
+@mpl3d_image_comparison(['trisurf3d.png'], tol=0.061, style='mpl20')
 def test_trisurf3d():
     n_angles = 36
     n_radii = 8
@@ -699,7 +746,7 @@ def test_trisurf3d():
     ax.plot_trisurf(x, y, z, cmap=cm.jet, linewidth=0.2)
 
 
-@mpl3d_image_comparison(['trisurf3d_shaded.png'], tol=0.03)
+@mpl3d_image_comparison(['trisurf3d_shaded.png'], tol=0.03, style='mpl20')
 def test_trisurf3d_shaded():
     n_angles = 36
     n_radii = 8
@@ -717,7 +764,7 @@ def test_trisurf3d_shaded():
     ax.plot_trisurf(x, y, z, color=[1, 0.5, 0], linewidth=0.2)
 
 
-@mpl3d_image_comparison(['wireframe3d.png'])
+@mpl3d_image_comparison(['wireframe3d.png'], style='mpl20')
 def test_wireframe3d():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -725,7 +772,7 @@ def test_wireframe3d():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=13)
 
 
-@mpl3d_image_comparison(['wireframe3dzerocstride.png'])
+@mpl3d_image_comparison(['wireframe3dzerocstride.png'], style='mpl20')
 def test_wireframe3dzerocstride():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -733,7 +780,7 @@ def test_wireframe3dzerocstride():
     ax.plot_wireframe(X, Y, Z, rcount=13, ccount=0)
 
 
-@mpl3d_image_comparison(['wireframe3dzerorstride.png'])
+@mpl3d_image_comparison(['wireframe3dzerorstride.png'], style='mpl20')
 def test_wireframe3dzerorstride():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -759,16 +806,25 @@ def test_mixedsamplesraises():
         ax.plot_surface(X, Y, Z, cstride=50, rcount=10)
 
 
-@mpl3d_image_comparison(
-    ['quiver3d.png', 'quiver3d_pivot_middle.png', 'quiver3d_pivot_tail.png'])
+@mpl3d_image_comparison(['quiver3d.png'], style='mpl20')
 def test_quiver3d():
-    x, y, z = np.ogrid[-1:0.8:10j, -1:0.8:10j, -1:0.6:3j]
-    u = np.sin(np.pi * x) * np.cos(np.pi * y) * np.cos(np.pi * z)
-    v = -np.cos(np.pi * x) * np.sin(np.pi * y) * np.cos(np.pi * z)
-    w = (2/3)**0.5 * np.cos(np.pi * x) * np.cos(np.pi * y) * np.sin(np.pi * z)
-    for pivot in ['tip', 'middle', 'tail']:
-        ax = plt.figure().add_subplot(projection='3d')
-        ax.quiver(x, y, z, u, v, w, length=0.1, pivot=pivot, normalize=True)
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='3d')
+    pivots = ['tip', 'middle', 'tail']
+    colors = ['tab:blue', 'tab:orange', 'tab:green']
+    for i, (pivot, color) in enumerate(zip(pivots, colors)):
+        x, y, z = np.meshgrid([-0.5, 0.5], [-0.5, 0.5], [-0.5, 0.5])
+        u = -x
+        v = -y
+        w = -z
+        # Offset each set in z direction
+        z += 2 * i
+        ax.quiver(x, y, z, u, v, w, length=1, pivot=pivot, color=color)
+        ax.scatter(x, y, z, color=color)
+
+    ax.set_xlim(-3, 3)
+    ax.set_ylim(-3, 3)
+    ax.set_zlim(-1, 5)
 
 
 @check_figures_equal(extensions=["png"])
@@ -779,7 +835,7 @@ def test_quiver3d_empty(fig_test, fig_ref):
     ax.quiver(x, y, z, u, v, w, length=0.1, pivot='tip', normalize=True)
 
 
-@mpl3d_image_comparison(['quiver3d_masked.png'])
+@mpl3d_image_comparison(['quiver3d_masked.png'], style='mpl20')
 def test_quiver3d_masked():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -848,7 +904,7 @@ def test_poly3dcollection_verts_validation():
         art3d.Poly3DCollection(poly)  # should be Poly3DCollection([poly])
 
 
-@mpl3d_image_comparison(['poly3dcollection_closed.png'])
+@mpl3d_image_comparison(['poly3dcollection_closed.png'], style='mpl20')
 def test_poly3dcollection_closed():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -878,7 +934,7 @@ def test_poly_collection_2d_to_3d_empty():
     fig.canvas.draw()
 
 
-@mpl3d_image_comparison(['poly3dcollection_alpha.png'])
+@mpl3d_image_comparison(['poly3dcollection_alpha.png'], style='mpl20')
 def test_poly3dcollection_alpha():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -897,7 +953,7 @@ def test_poly3dcollection_alpha():
     ax.add_collection3d(c2)
 
 
-@mpl3d_image_comparison(['add_collection3d_zs_array.png'])
+@mpl3d_image_comparison(['add_collection3d_zs_array.png'], style='mpl20')
 def test_add_collection3d_zs_array():
     theta = np.linspace(-4 * np.pi, 4 * np.pi, 100)
     z = np.linspace(-2, 2, 100)
@@ -925,7 +981,7 @@ def test_add_collection3d_zs_array():
     ax.set_zlim(-2, 2)
 
 
-@mpl3d_image_comparison(['add_collection3d_zs_scalar.png'])
+@mpl3d_image_comparison(['add_collection3d_zs_scalar.png'], style='mpl20')
 def test_add_collection3d_zs_scalar():
     theta = np.linspace(0, 2 * np.pi, 100)
     z = 1
@@ -951,7 +1007,8 @@ def test_add_collection3d_zs_scalar():
     ax.set_zlim(0, 2)
 
 
-@mpl3d_image_comparison(['axes3d_labelpad.png'], remove_text=False)
+@mpl3d_image_comparison(['axes3d_labelpad.png'],
+                        remove_text=False, style='mpl20')
 def test_axes3d_labelpad():
     fig = plt.figure()
     ax = fig.add_axes(Axes3D(fig))
@@ -973,7 +1030,7 @@ def test_axes3d_labelpad():
         tick.set_pad(tick.get_pad() - i * 5)
 
 
-@mpl3d_image_comparison(['axes3d_cla.png'], remove_text=False)
+@mpl3d_image_comparison(['axes3d_cla.png'], remove_text=False, style='mpl20')
 def test_axes3d_cla():
     # fixed in pull request 4553
     fig = plt.figure()
@@ -982,7 +1039,8 @@ def test_axes3d_cla():
     ax.cla()  # make sure the axis displayed is 3D (not 2D)
 
 
-@mpl3d_image_comparison(['axes3d_rotated.png'], remove_text=False)
+@mpl3d_image_comparison(['axes3d_rotated.png'],
+                        remove_text=False, style='mpl20')
 def test_axes3d_rotated():
     fig = plt.figure()
     ax = fig.add_subplot(1, 1, 1, projection='3d')
@@ -1009,20 +1067,21 @@ def _test_proj_make_M():
     roll = 0
     u, v, w = proj3d._view_axes(E, R, V, roll)
     viewM = proj3d._view_transformation_uvw(u, v, w, E)
-    perspM = proj3d.persp_transformation(100, -100, 1)
+    perspM = proj3d._persp_transformation(100, -100, 1)
     M = np.dot(perspM, viewM)
     return M
 
 
 def test_proj_transform():
     M = _test_proj_make_M()
+    invM = np.linalg.inv(M)
 
     xs = np.array([0, 1, 1, 0, 0, 0, 1, 1, 0, 0]) * 300.0
     ys = np.array([0, 0, 1, 1, 0, 0, 0, 1, 1, 0]) * 300.0
     zs = np.array([0, 0, 0, 0, 0, 1, 1, 1, 1, 1]) * 300.0
 
     txs, tys, tzs = proj3d.proj_transform(xs, ys, zs, M)
-    ixs, iys, izs = proj3d.inv_transform(txs, tys, tzs, M)
+    ixs, iys, izs = proj3d.inv_transform(txs, tys, tzs, invM)
 
     np.testing.assert_almost_equal(ixs, xs)
     np.testing.assert_almost_equal(iys, ys)
@@ -1046,7 +1105,7 @@ def _test_proj_draw_axes(M, s=1, *args, **kwargs):
     return fig, ax
 
 
-@mpl3d_image_comparison(['proj3d_axes_cube.png'])
+@mpl3d_image_comparison(['proj3d_axes_cube.png'], style='mpl20')
 def test_proj_axes_cube():
     M = _test_proj_make_M()
 
@@ -1068,7 +1127,7 @@ def test_proj_axes_cube():
     ax.set_ylim(-0.2, 0.2)
 
 
-@mpl3d_image_comparison(['proj3d_axes_cube_ortho.png'])
+@mpl3d_image_comparison(['proj3d_axes_cube_ortho.png'], style='mpl20')
 def test_proj_axes_cube_ortho():
     E = np.array([200, 100, 100])
     R = np.array([0, 0, 0])
@@ -1076,7 +1135,7 @@ def test_proj_axes_cube_ortho():
     roll = 0
     u, v, w = proj3d._view_axes(E, R, V, roll)
     viewM = proj3d._view_transformation_uvw(u, v, w, E)
-    orthoM = proj3d.ortho_transformation(-1, 1)
+    orthoM = proj3d._ortho_transformation(-1, 1)
     M = np.dot(orthoM, viewM)
 
     ts = '0 1 2 3 0 4 5 6 7 4'.split()
@@ -1097,16 +1156,6 @@ def test_proj_axes_cube_ortho():
     ax.set_ylim(-200, 200)
 
 
-def test_rot():
-    V = [1, 0, 0, 1]
-    rotated_V = proj3d.rot_x(V, np.pi / 6)
-    np.testing.assert_allclose(rotated_V, [1, 0, 0, 1])
-
-    V = [0, 1, 0, 1]
-    rotated_V = proj3d.rot_x(V, np.pi / 6)
-    np.testing.assert_allclose(rotated_V, [0, np.sqrt(3) / 2, 0.5, 1])
-
-
 def test_world():
     xmin, xmax = 100, 120
     ymin, ymax = -100, 100
@@ -1119,39 +1168,6 @@ def test_world():
                                 [0, 0, 0, 1]])
 
 
-@mpl3d_image_comparison(['proj3d_lines_dists.png'])
-def test_lines_dists():
-    fig, ax = plt.subplots(figsize=(4, 6), subplot_kw=dict(aspect='equal'))
-
-    xs = (0, 30)
-    ys = (20, 150)
-    ax.plot(xs, ys)
-    p0, p1 = zip(xs, ys)
-
-    xs = (0, 0, 20, 30)
-    ys = (100, 150, 30, 200)
-    ax.scatter(xs, ys)
-
-    dist0 = proj3d._line2d_seg_dist((xs[0], ys[0]), p0, p1)
-    dist = proj3d._line2d_seg_dist(np.array((xs, ys)).T, p0, p1)
-    assert dist0 == dist[0]
-
-    for x, y, d in zip(xs, ys, dist):
-        c = Circle((x, y), d, fill=0)
-        ax.add_patch(c)
-
-    ax.set_xlim(-50, 150)
-    ax.set_ylim(0, 300)
-
-
-def test_lines_dists_nowarning():
-    # No RuntimeWarning must be emitted for degenerate segments, see GH#22624.
-    s0 = (10, 30, 50)
-    p = (20, 150, 180)
-    proj3d._line2d_seg_dist(p, s0, s0)
-    proj3d._line2d_seg_dist(np.array(p), s0, s0)
-
-
 def test_autoscale():
     fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
     assert ax.get_zscale() == 'linear'
@@ -1198,21 +1214,22 @@ def test_axes3d_focal_length_checks():
         ax.set_proj_type('ortho', focal_length=1)
 
 
-@mpl3d_image_comparison(['axes3d_focal_length.png'], remove_text=False)
+@mpl3d_image_comparison(['axes3d_focal_length.png'],
+                        remove_text=False, style='mpl20')
 def test_axes3d_focal_length():
     fig, axs = plt.subplots(1, 2, subplot_kw={'projection': '3d'})
     axs[0].set_proj_type('persp', focal_length=np.inf)
     axs[1].set_proj_type('persp', focal_length=0.15)
 
 
-@mpl3d_image_comparison(['axes3d_ortho.png'], remove_text=False)
+@mpl3d_image_comparison(['axes3d_ortho.png'], remove_text=False, style='mpl20')
 def test_axes3d_ortho():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
     ax.set_proj_type('ortho')
 
 
-@mpl3d_image_comparison(['axes3d_isometric.png'])
+@mpl3d_image_comparison(['axes3d_isometric.png'], style='mpl20')
 def test_axes3d_isometric():
     from itertools import combinations, product
     fig, ax = plt.subplots(subplot_kw=dict(
@@ -1246,7 +1263,7 @@ def test_invalid_axes_limits(setter, side, value):
 
 
 class TestVoxels:
-    @mpl3d_image_comparison(['voxels-simple.png'])
+    @mpl3d_image_comparison(['voxels-simple.png'], style='mpl20')
     def test_simple(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -1254,7 +1271,7 @@ class TestVoxels:
         voxels = (x == y) | (y == z)
         ax.voxels(voxels)
 
-    @mpl3d_image_comparison(['voxels-edge-style.png'])
+    @mpl3d_image_comparison(['voxels-edge-style.png'], style='mpl20')
     def test_edge_style(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -1265,7 +1282,7 @@ class TestVoxels:
         # change the edge color of one voxel
         v[max(v.keys())].set_edgecolor('C2')
 
-    @mpl3d_image_comparison(['voxels-named-colors.png'])
+    @mpl3d_image_comparison(['voxels-named-colors.png'], style='mpl20')
     def test_named_colors(self):
         """Test with colors set to a 3D object array of strings."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -1278,7 +1295,7 @@ class TestVoxels:
         colors[(x + z) < 10] = 'cyan'
         ax.voxels(voxels, facecolors=colors)
 
-    @mpl3d_image_comparison(['voxels-rgb-data.png'])
+    @mpl3d_image_comparison(['voxels-rgb-data.png'], style='mpl20')
     def test_rgb_data(self):
         """Test with colors set to a 4d float array of rgb data."""
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
@@ -1291,7 +1308,7 @@ class TestVoxels:
         colors[..., 2] = z / 9
         ax.voxels(voxels, facecolors=colors)
 
-    @mpl3d_image_comparison(['voxels-alpha.png'])
+    @mpl3d_image_comparison(['voxels-alpha.png'], style='mpl20')
     def test_alpha(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -1309,7 +1326,8 @@ class TestVoxels:
             assert voxels[coord], "faces returned for absent voxel"
             assert isinstance(poly, art3d.Poly3DCollection)
 
-    @mpl3d_image_comparison(['voxels-xyz.png'], tol=0.01, remove_text=False)
+    @mpl3d_image_comparison(['voxels-xyz.png'],
+                            tol=0.01, remove_text=False, style='mpl20')
     def test_xyz(self):
         fig, ax = plt.subplots(subplot_kw={"projection": "3d"})
 
@@ -1501,7 +1519,7 @@ def test_minor_ticks():
     ax.set_zticklabels(["half"], minor=True)
 
 
-@mpl3d_image_comparison(['errorbar3d_errorevery.png'])
+@mpl3d_image_comparison(['errorbar3d_errorevery.png'], style='mpl20')
 def test_errorbar3d_errorevery():
     """Tests errorevery functionality for 3D errorbars."""
     t = np.arange(0, 2*np.pi+.1, 0.01)
@@ -1519,7 +1537,7 @@ def test_errorbar3d_errorevery():
                 errorevery=estep)
 
 
-@mpl3d_image_comparison(['errorbar3d.png'])
+@mpl3d_image_comparison(['errorbar3d.png'], style='mpl20')
 def test_errorbar3d():
     """Tests limits, color styling, and legend for 3D errorbars."""
     fig = plt.figure()
@@ -1535,8 +1553,7 @@ def test_errorbar3d():
     ax.legend()
 
 
-@image_comparison(['stem3d.png'], style='mpl20',
-                  tol=0.003)
+@image_comparison(['stem3d.png'], style='mpl20', tol=0.003)
 def test_stem3d():
     fig, axs = plt.subplots(2, 3, figsize=(8, 6),
                             constrained_layout=True,
@@ -1668,6 +1685,20 @@ def test_set_zlim():
         ax.set_zlim(top=0, zmax=1)
 
 
+@check_figures_equal(extensions=["png"])
+def test_shared_view(fig_test, fig_ref):
+    elev, azim, roll = 5, 20, 30
+    ax1 = fig_test.add_subplot(131, projection="3d")
+    ax2 = fig_test.add_subplot(132, projection="3d", shareview=ax1)
+    ax3 = fig_test.add_subplot(133, projection="3d")
+    ax3.shareview(ax1)
+    ax2.view_init(elev=elev, azim=azim, roll=roll, share=True)
+
+    for subplot_num in (131, 132, 133):
+        ax = fig_ref.add_subplot(subplot_num, projection="3d")
+        ax.view_init(elev=elev, azim=azim, roll=roll)
+
+
 def test_shared_axes_retick():
     fig = plt.figure()
     ax1 = fig.add_subplot(211, projection="3d")
@@ -1798,9 +1829,9 @@ def test_toolbar_zoom_pan(tool, button, key, expected):
 @check_figures_equal(extensions=["png"])
 def test_scalarmap_update(fig_test, fig_ref):
 
-    x, y, z = np.array((list(itertools.product(*[np.arange(0, 5, 1),
-                                                 np.arange(0, 5, 1),
-                                                 np.arange(0, 5, 1)])))).T
+    x, y, z = np.array(list(itertools.product(*[np.arange(0, 5, 1),
+                                                np.arange(0, 5, 1),
+                                                np.arange(0, 5, 1)]))).T
     c = x + y
 
     # test
@@ -1826,7 +1857,7 @@ def test_subfigure_simple():
 
 # Update style when regenerating the test image
 @image_comparison(baseline_images=['computed_zorder'], remove_text=True,
-                  extensions=['png'], style=('classic', '_classic_test_patch'))
+                  extensions=['png'], style=('mpl20'))
 def test_computed_zorder():
     fig = plt.figure()
     ax1 = fig.add_subplot(221, projection='3d')
@@ -1913,16 +1944,30 @@ def test_format_coord():
     ax = fig.add_subplot(projection='3d')
     x = np.arange(10)
     ax.plot(x, np.sin(x))
+    xv = 0.1
+    yv = 0.1
     fig.canvas.draw()
-    assert ax.format_coord(0, 0) == 'x=1.8066, y=1.0367, z=−0.0553'
+    assert ax.format_coord(xv, yv) == 'x=10.5227, y pane=1.0417, z=0.1444'
+
     # Modify parameters
     ax.view_init(roll=30, vertical_axis="y")
     fig.canvas.draw()
-    assert ax.format_coord(0, 0) == 'x=9.1651, y=−0.9215, z=−0.0359'
+    assert ax.format_coord(xv, yv) == 'x pane=9.1875, y=0.9761, z=0.1291'
+
     # Reset parameters
     ax.view_init()
     fig.canvas.draw()
-    assert ax.format_coord(0, 0) == 'x=1.8066, y=1.0367, z=−0.0553'
+    assert ax.format_coord(xv, yv) == 'x=10.5227, y pane=1.0417, z=0.1444'
+
+    # Check orthographic projection
+    ax.set_proj_type('ortho')
+    fig.canvas.draw()
+    assert ax.format_coord(xv, yv) == 'x=10.8869, y pane=1.0417, z=0.1528'
+
+    # Check non-default perspective projection
+    ax.set_proj_type('persp', focal_length=0.1)
+    fig.canvas.draw()
+    assert ax.format_coord(xv, yv) == 'x=9.0620, y pane=1.0417, z=0.1110'
 
 
 def test_get_axis_position():
@@ -2007,7 +2052,7 @@ def test_pathpatch_3d(fig_test, fig_ref):
 
 @image_comparison(baseline_images=['scatter_spiral.png'],
                   remove_text=True,
-                  style='default')
+                  style='mpl20')
 def test_scatter_spiral():
     fig = plt.figure()
     ax = fig.add_subplot(projection='3d')
@@ -2126,13 +2171,13 @@ def test_view_init_vertical_axis(
 
         # Assert ticks are correctly aligned:
         tickdir_expected = tickdirs_expected[i]
-        tickdir_actual = axis._get_tickdir()
+        tickdir_actual = axis._get_tickdir('default')
         np.testing.assert_array_equal(tickdir_expected, tickdir_actual)
 
 
 @image_comparison(baseline_images=['arc_pathpatch.png'],
                   remove_text=True,
-                  style='default')
+                  style='mpl20')
 def test_arc_pathpatch():
     ax = plt.subplot(1, 1, 1, projection="3d")
     a = mpatch.Arc((0.5, 0.5), width=0.5, height=0.9,
@@ -2176,3 +2221,61 @@ def test_mutating_input_arrays_y_and_z(fig_test, fig_ref):
     y = [0.0, 0.0, 0.0]
     z = [0.0, 0.0, 0.0]
     ax2.plot(x, y, z, 'o-')
+
+
+def test_scatter_masked_color():
+    """
+    Test color parameter usage with non-finite coordinate arrays.
+
+    GH#26236
+    """
+
+    x = [np.nan, 1, 2,  1]
+    y = [0, np.inf, 2,  1]
+    z = [0, 1, -np.inf, 1]
+    colors = [
+        [0.0, 0.0, 0.0, 1],
+        [0.0, 0.0, 0.0, 1],
+        [0.0, 0.0, 0.0, 1],
+        [0.0, 0.0, 0.0, 1]
+    ]
+
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='3d')
+    path3d = ax.scatter(x, y, z, color=colors)
+
+    # Assert sizes' equality
+    assert len(path3d.get_offsets()) ==\
+           len(super(type(path3d), path3d).get_facecolors())
+
+
+@mpl3d_image_comparison(['surface3d_zsort_inf.png'], style='mpl20')
+def test_surface3d_zsort_inf():
+    fig = plt.figure()
+    ax = fig.add_subplot(projection='3d')
+
+    x, y = np.mgrid[-2:2:0.1, -2:2:0.1]
+    z = np.sin(x)**2 + np.cos(y)**2
+    z[x.shape[0] // 2:, x.shape[1] // 2:] = np.inf
+
+    ax.plot_surface(x, y, z, cmap='jet')
+    ax.view_init(elev=45, azim=145)
+
+
+def test_Poly3DCollection_init_value_error():
+    # smoke test to ensure the input check works
+    # GH#26420
+    with pytest.raises(ValueError,
+                       match='You must provide facecolors, edgecolors, '
+                        'or both for shade to work.'):
+        poly = np.array([[0, 0, 1], [0, 1, 1], [0, 0, 0]], float)
+        c = art3d.Poly3DCollection([poly], shade=True)
+
+
+def test_ndarray_color_kwargs_value_error():
+    # smoke test
+    # ensures ndarray can be passed to color in kwargs for 3d projection plot
+    fig = plt.figure()
+    ax = fig.add_subplot(111, projection='3d')
+    ax.scatter(1, 0, 0, color=np.array([0, 0, 0, 1]))
+    fig.canvas.draw()
